@@ -2,6 +2,12 @@ import requests
 from .utils import *
 
 
+def test_blame():
+    with serve():
+        response = requests.get(UNAUTH_TEST_REPO_URL + "blob/HEAD/test.c")
+        assert response.status_code == 200
+
+
 def test_dont_show_blame_link():
     with serve():
         for file in ["binary", "image.jpg", "toolarge"]:
